Number of digits after the decimal separator
CREATE TABLE currency
(


INSERT INTO currency(code, num, decimal_digit, name) VALUES


('AED', '784', 2, 'United Arab Emirates dirham'),
('AFN', '971', 2, 'Afghan afghani'),
('ALL', '008', 2, 'Albanian lek'),
('AMD', '051', 2, 'Armenian dram'),
('ANG', '532', 2, 'Netherlands Antillean guilder'),
('AOA', '973', 2, 'Angolan kwanza'),
('ARS', '032', 2, 'Argentine peso'),
('AUD', '036', 2, 'Australian dollar'),
('AWG', '533', 2, 'Aruban florin'),
('AZN', '944', 2, 'Azerbaijani manat'),
('BAM', '977', 2, 'Bosnia and Herzegovina convertible mark'),
('BBD', '052', 2, 'Barbados dollar'),
('BDT', '050', 2, 'Bangladeshi taka'),
('BGN', '975', 2, 'Bulgarian lev'),
('BHD', '048', 3, 'Bahraini dinar'),
('BIF', '108', 0, 'Burundian franc'),
('BMD', '060', 2, 'Bermudian dollar'),
('BND', '096', 2, 'Brunei dollar'),
('BOB', '068', 2, 'Boliviano'),
('BOV', '984', 2, 'Bolivian Mvdol (funds code)     Bolivia
BRL'),
('BSD', '044', 2, 'Bahamian dollar'),
('BTN', '064', 2, 'Bhutanese ngultrum'),
('BWP', '072', 2, 'Botswana pula'),
('BYR', '974', 0, 'Belarusian ruble'),
('BZD', '084', 2, 'Belize dollar'),
('CAD', '124', 2, 'Canadian dollar'),
('CDF', '976', 2, 'Congolese franc'),
('CHE', '947', 2, 'WIR Euro (complementary currency)'),
('CHF', '756', 2, 'Swiss franc'),
('CHW', '948', 2, 'WIR Franc (complementary currency)'),
('CLF', '990', 0, 'Unidad de Fomento (funds code)'),
('CLP', '152', 0, 'Chilean peso'),
('CNY', '156', 2, 'Chinese yuan'),
('COP', '170', 2, 'Colombian peso'),
('COU', '970', 2, 'Unidad de Valor Real  Colombia
CRC'),
('CUC', '931', 2, 'Cuban convertible peso  Cuba
CUP'),
('CVE', '132', 0, 'Cape Verde escudo   Cape Verde
CZK'),
('DJF', '262', 0, 'Djiboutian franc      Djibouti
DKK'),
('DOP', '214', 2, 'Dominican peso'),
('DZD', '012', 2, 'Algerian dinar'),
('EGP', '818', 2, 'Egyptian pound'),
('ERN', '232', 2, 'Eritrean nakfa'),
('ETB', '230', 2, 'Ethiopian birr'),
('EUR', '978', 2, 'Euro'),
('FJD', '242', 2, 'Fiji dollar'),
('FKP', '238', 2, 'Falkland Islands pound'),
('GBP', '826', 2, 'Pound sterling'),
('GEL', '981', 2, 'Georgian lari'),
('GHS', '936', 2, 'Ghanaian cedi'),
('GIP', '292', 2, 'Gibraltar pound'),
('GMD', '270', 2, 'Gambian dalasi'),
('GNF', '324', 0, 'Guinean franc'),
('GTQ', '320', 2, 'Guatemalan quetzal      Guatemala
GYD'),
('HKD', '344', 2, 'Hong Kong dollar'),
('HNL', '340', 2, 'Honduran lempira'),
('HRK', '191', 2, 'Croatian kuna'),
('HTG', '332', 2, 'Haitian gourde'),
('HUF', '348', 2, 'Hungarian forint'),
('IDR', '360', 0, 'Indonesian rupiah'),
('ILS', '376', 2, 'Israeli new shekel'),
('INR', '356', 2, 'Indian rupee'),
('IQD', '368', 3, 'Iraqi dinar'),
('IRR', '364', 0, 'Iranian rial'),
('ISK', '352', 0, 'Icelandic króna'),
('JMD', '388', 2, 'Jamaican dollar'),
('JOD', '400', 3, 'Jordanian dinar'),
('JPY', '392', 0, 'Japanese yen'),
('KES', '404', 2, 'Kenyan shilling'),
('KGS', '417', 2, 'Kyrgyzstani som'),
('KHR', '116', 2, 'Cambodian riel'),
('KMF', '174', 0, 'Comoro franc'),
('KPW', '408', 0, 'North Korean won'),
('KRW', '410', 0, 'South Korean won'),
('KWD', '414', 3, 'Kuwaiti dinar'),
('KYD', '136', 2, 'Cayman Islands dollar'),
('KZT', '398', 2, 'Kazakhstani tenge'),
('LAK', '418', 0, 'Lao kip'),
('LBP', '422', 0, 'Lebanese pound'),
('LKR', '144', 2, 'Sri Lankan rupee  Sri Lanka
LRD'),
('LSL', '426', 2, 'Lesotho loti'),
('LTL', '440', 2, 'Lithuanian litas     Lithuania
LVL'),
('LYD', '434', 3, 'Libyan dinar'),
('MAD', '504', 2, 'Moroccan dirham'),
('MDL', '498', 2, 'Moldovan leu'),
('MGA', '969', 0*[8], 'Malagasy ariary'),
('MKD', '807', 0, 'Macedonian denar'),
('MMK', '104', 0, 'Myanma kyat'),
('MNT', '496', 2, 'Mongolian tugrik'),
('MOP', '446', 2, 'Macanese pataca'),
('MRO', '478', 0*[8], 'Mauritanian ouguiya'),
('MUR', '480', 2, 'Mauritian rupee'),
('MVR', '462', 2, 'MaldivianEN'),
('PGK', '598', 2, 'Papua New Guinean kina'),
('PHP', '608', 2, 'Philippine peso'),
('PKR', '586', 2, 'Pakistani rupee'),
('PLN', '985', 2, 'Polish złoty'),
('PYG', '600', 0, 'Paraguayan guaraní    Paraguay
QAR'),
('RON', '946', 2, 'Romanian new leu        Romania
RSD'),
('TJS', '972', 2, 'Tajikistani somoni'),
('TMT', '934', 2, 'Turkmenistani manat'),
('TND', '788', 3, 'Tunisian dinar'),
('TOP', '776', 2, 'Tongan paʻanga'),
('TRY', '949', 2, 'Turkish lira'),
('TTD', '780', 2, 'Trinidad and Tobago dollar     Trinidad and Tobago
TWD'),
('UYI', '940', 0, 'Uruguay Peso en Unidades Indexadas (URUIURUI) (funds code)   Uruguay
UYU'),
('UZS', '860', 2, 'Uzbekistan som'),
('VEF', '937', 2, 'Venezuelan bolívar fuerte'),
('VND', '704', 0, 'Vietnamese dSpecial drawing rights'),
('XFU', 'Nil', ., 'UIC franc (special settlement currency)'),
('XOF', '952', 0, 'CFA franc BCEAO'),
('XPD', '964', ., 'Palladium (one troy ounce)  
XPF'),
('XPT', '962', ., 'Platinum (one troy ounce)'),
('XTS', '963', ., 'Code reserved for testing purposes'),
('XXX', '999', ., 'No currency'